@@ -13,8 +13,10 @@ import ctypes
 import functools
 import logging
 import sys
+import time
 import warnings
-from typing import Callable, List, Optional, Sequence, Tuple, Union
+from collections.abc import Sequence
+from typing import Callable, Optional, Union
 
 from can import (
     BusABC,
@@ -35,11 +37,11 @@ from . import constants, structures
 from .exceptions import *
 
 __all__ = [
-    "VCITimeout",
-    "VCIError",
+    "IXXATBus",
     "VCIBusOffError",
     "VCIDeviceNotFoundError",
-    "IXXATBus",
+    "VCIError",
+    "VCITimeout",
     "vciFormatError",
 ]
 
@@ -64,7 +66,7 @@ else:
 
 
 def __vciFormatErrorExtended(
-    library_instance: CLibrary, function: Callable, vret: int, args: Tuple
+    library_instance: CLibrary, function: Callable, vret: int, args: tuple
 ):
     """Format a VCI error and attach failed function, decoded HRESULT and arguments
     :param CLibrary library_instance:
@@ -619,7 +621,15 @@ class IXXATBus(BusABC):
                 log.info("Accepting ID: 0x%X MASK: 0x%X", code, mask)
 
         # Start the CAN controller. Messages will be forwarded to the channel
+        start_begin = time.time()
         _canlib.canControlStart(self._control_handle, constants.TRUE)
+        start_end = time.time()
+
+        # Calculate an offset to make them relative to epoch
+        # Assume that the time offset is in the middle of the start command
+        self._timeoffset = start_begin + (start_end - start_begin / 2)
+        self._overrunticks = 0
+        self._starttickoffset = 0
 
         # For cyclic transmit list. Set when .send_periodic() is first called
         self._scheduler = None
@@ -692,6 +702,9 @@ class IXXATBus(BusABC):
                         f"Unknown CAN info message code {self._message.abData[0]}",
                     )
                 )
+                # Handle CAN start info message
+                if self._message.abData[0] == constants.CAN_INFO_START:
+                    self._starttickoffset = self._message.dwTime
             elif self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_ERROR:
                 if self._message.uMsgInfo.Bytes.bFlags & constants.CAN_MSGFLAGS_OVR:
                     log.warning("CAN error: data overrun")
@@ -708,7 +721,8 @@ class IXXATBus(BusABC):
                         self._message.uMsgInfo.Bytes.bFlags,
                     )
             elif self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_TIMEOVR:
-                pass
+                # Add the number of timestamp overruns to the high word
+                self._overrunticks += self._message.dwMsgId << 32
             else:
                 log.warning(
                     "Unexpected message info type 0x%X",
@@ -740,11 +754,12 @@ class IXXATBus(BusABC):
             # Timed out / can message type is not DATA
             return None, True
 
-        # The _message.dwTime is a 32bit tick value and will overrun,
-        # so expect to see the value restarting from 0
         rx_msg = Message(
-            timestamp=self._message.dwTime
-            / self._tick_resolution,  # Relative time in s
+            timestamp=(
+                (self._message.dwTime + self._overrunticks - self._starttickoffset)
+                / self._tick_resolution
+            )
+            + self._timeoffset,
             is_remote_frame=bool(self._message.uMsgInfo.Bits.rtr),
             is_extended_id=bool(self._message.uMsgInfo.Bits.ext),
             arbitration_id=self._message.dwMsgId,
@@ -793,6 +808,7 @@ class IXXATBus(BusABC):
         msgs: Union[Sequence[Message], Message],
         period: float,
         duration: Optional[float] = None,
+        autostart: bool = True,
         modifier_callback: Optional[Callable[[Message], None]] = None,
     ) -> CyclicSendTaskABC:
         """Send a message using built-in cyclic transmit list functionality."""
@@ -807,7 +823,12 @@ class IXXATBus(BusABC):
                 self._scheduler_resolution = caps.dwClockFreq / caps.dwCmsDivisor
                 _canlib.canSchedulerActivate(self._scheduler, constants.TRUE)
             return CyclicSendTask(
-                self._scheduler, msgs, period, duration, self._scheduler_resolution
+                self._scheduler,
+                msgs,
+                period,
+                duration,
+                self._scheduler_resolution,
+                autostart=autostart,
             )
 
         # fallback to thread based cyclic task
@@ -821,6 +842,7 @@ class IXXATBus(BusABC):
             msgs=msgs,
             period=period,
             duration=duration,
+            autostart=autostart,
             modifier_callback=modifier_callback,
         )
 
@@ -863,7 +885,15 @@ class IXXATBus(BusABC):
 class CyclicSendTask(LimitedDurationCyclicSendTaskABC, RestartableCyclicTaskABC):
     """A message in the cyclic transmit list."""
 
-    def __init__(self, scheduler, msgs, period, duration, resolution):
+    def __init__(
+        self,
+        scheduler,
+        msgs,
+        period,
+        duration,
+        resolution,
+        autostart: bool = True,
+    ):
         super().__init__(msgs, period, duration)
         if len(self.messages) != 1:
             raise ValueError(
@@ -875,7 +905,7 @@ class CyclicSendTask(LimitedDurationCyclicSendTaskABC, RestartableCyclicTaskABC)
         self._count = int(duration / period) if duration else 0
 
         self._msg = structures.CANCYCLICTXMSG()
-        self._msg.wCycleTime = int(round(period * resolution))
+        self._msg.wCycleTime = round(period * resolution)
         self._msg.dwMsgId = self.messages[0].arbitration_id
         self._msg.uMsgInfo.Bits.type = constants.CAN_MSGTYPE_DATA
         self._msg.uMsgInfo.Bits.ext = 1 if self.messages[0].is_extended_id else 0
@@ -883,7 +913,8 @@ class CyclicSendTask(LimitedDurationCyclicSendTaskABC, RestartableCyclicTaskABC)
         self._msg.uMsgInfo.Bits.dlc = self.messages[0].dlc
         for i, b in enumerate(self.messages[0].data):
             self._msg.abData[i] = b
-        self.start()
+        if autostart:
+            self.start()
 
     def start(self):
         """Start transmitting message (add to list if needed)."""
@@ -945,8 +976,8 @@ def get_ixxat_hwids():
     return hwids
 
 
-def _detect_available_configs() -> List[AutoDetectedConfig]:
-    config_list = []  # list in wich to store the resulting bus kwargs
+def _detect_available_configs() -> Sequence["AutoDetectedIxxatConfig"]:
+    config_list = []  # list in which to store the resulting bus kwargs
 
     # used to detect HWID
     device_handle = HANDLE()
@@ -995,3 +1026,7 @@ def _detect_available_configs() -> List[AutoDetectedConfig]:
         pass  # _canlib is None in the CI tests -> return a blank list
 
     return config_list
+
+
+class AutoDetectedIxxatConfig(AutoDetectedConfig):
+    unique_hardware_id: int
